# -*- coding: UTF-8 -*-
import os

from mock import Mock
from nose.tools import raises

import oca
import oca.pool


class TestVirtualMachine:
    def setUp(self):
        self.client = oca.Client('test:test')
        self.xml = open(os.path.join(os.path.dirname(oca.__file__),
                                     'tests/fixtures/vm.xml')).read()

    def test_new_with_id(self):
        vm = oca.VirtualMachine.new_with_id(self.client, 1)
        assert vm.id == 1

    def test_acces_items_using_brackets(self):
        vm = oca.VirtualMachine(self.xml, self.client)
        assert vm['name'] == 'vm-example'
        assert vm['ID'] == '6'
        assert vm['last_poll'] == '1277729095'
        assert vm['state'] == '3'
        assert vm['lcm_state'] == '3'
        assert vm['stime'] == '1277375180'
        assert vm['etime'] == '0'
        assert vm['deploy_id'] == 'dummy'

    def test_acces_items_not_using_brackets(self):
        vm = oca.VirtualMachine(self.xml, self.client)
        assert vm.name == 'vm-example'
        assert vm.ID == '6'
        assert vm.last_poll == '1277729095'
        assert vm.state == '3'
        assert vm.lcm_state == '3'
        assert vm.stime == '1277375180'
        assert vm.etime == '0'
        assert vm.deploy_id == 'dummy'

    @raises(IndexError)
    def test_raise_exception_Index_Error_when_using_brackets(self):
        vm = oca.VirtualMachine(self.xml, self.client)
        vm['wrong_name']

    def test_convert_types(self):
        vm = oca.VirtualMachine(self.xml, self.client)
        vm._convert_types()
        assert vm.name == 'vm-example'
        assert vm.id == 6
        assert vm.last_poll == 1277729095
        assert vm.state == 3
        assert vm.lcm_state == 3
        assert vm.stime == 1277375180
        assert vm.etime == 0
        assert vm.deploy_id == 'dummy'
        assert isinstance(vm.template, oca.pool.Template)

    def test_allocate(self):
        self.client.call = Mock(return_value=3)
        assert oca.VirtualMachine.allocate(self.client, '<VM></VM>') == 3

    def test_deploy(self):
        self.client.call = Mock(return_value='')
        vm = oca.VirtualMachine(self.xml, self.client)
        vm.deploy(3)
        self.client.call.assert_called_once_with('vm.deploy', '6', 3)

    def test_migrate(self):
        self.client.call = Mock(return_value='')
        vm = oca.VirtualMachine(self.xml, self.client)
        vm.migrate(3)
        self.client.call.assert_called_once_with('vm.migrate', '6', 3, False)

    def test_live_migrate(self):
        self.client.call = Mock(return_value='')
        vm = oca.VirtualMachine(self.xml, self.client)
        vm.live_migrate(3)
        self.client.call.assert_called_once_with('vm.migrate', '6', 3, True)

    def test_save_disk(self):
        self.client.call = Mock(return_value='')
        vm = oca.VirtualMachine(self.xml, self.client)
        vm.save_disk(1, 2)
        self.client.call.assert_called_once_with('vm.savedisk', '6', 1, 2)

    def test_actions(self):
        oca.client = oca.Client('test:test')
        vm = oca.VirtualMachine(self.xml, self.client)
        for action in ['shutdown', 'hold', 'release', 'stop', 'cancel',
                'suspend', 'resume', 'restart', 'finalize', 'delete']:
            self.client.call = Mock(return_value='')
            getattr(vm, action)()
            self.client.call.assert_called_once_with('vm.action', action, '6')

    def test_repr(self):
        vm = oca.VirtualMachine(self.xml, self.client)
        assert vm.__repr__() == '<oca.VirtualMachine("vm-example")>'

    def test_states(self):
        for i in range(len(oca.VirtualMachine.VM_STATE)):
            vm = oca.VirtualMachine('<VM><ID>2</ID><STATE>%s</STATE></VM>' % i,
                                self.client)
            assert vm.str_state == oca.VirtualMachine.VM_STATE[i]
            state = oca.VirtualMachine.SHORT_VM_STATES[oca.VirtualMachine.VM_STATE[i]]
            assert vm.short_state == state

    def test_lcm_states(self):
        for i in range(len(oca.VirtualMachine.LCM_STATE)):
            xml = '<VM><ID>2</ID><LCM_STATE>%s</LCM_STATE></VM>' % i
            vm = oca.VirtualMachine(xml, self.client)
            assert vm.str_lcm_state == oca.VirtualMachine.LCM_STATE[i]
            lcm = oca.VirtualMachine.SHORT_LCM_STATES[oca.VirtualMachine.LCM_STATE[i]]
            assert vm.short_lcm_state == lcm

    def test_resubmit(self):
        self.client.call = Mock(return_value='')
        vm = oca.VirtualMachine(self.xml, self.client)
        vm.resubmit()
        self.client.call.assert_called_once_with('vm.action', 'resubmit', '6')

    def test_History_repr(self):
        vm = oca.VirtualMachine(self.xml, self.client)
        vm._convert_types()
        history = vm.history_records[0]
        assert repr(history) == '<oca.vm.History("seq=0")>'

    def test_no_history_records_element(self):
        vm = oca.VirtualMachine(self.xml, self.client)
        vm.xml.remove(vm.xml.find('HISTORY_RECORDS'))
        vm._convert_types()
        assert vm.history_records == []

    def test_user_template_variables(self):
        vm = oca.VirtualMachine(self.xml, self.client)
        vm._convert_types()
        greeting = vm.user_template.greeting
        assert greeting == "Hello World"

    def test_update(self):
        pass



# -*- coding: UTF-8 -*-
import os

from mock import Mock

import oca


VN_TEMPLATE = '''NAME = "Red LAN"
TYPE = RANGED
PUBLIC = NO
BRIDGE = vbr0
NETWORK_SIZE    = C
NETWORK_ADDRESS = 192.168.0.0'''


class TestVirtualNetwork:
    def setUp(self):
        self.client = oca.Client('test:test')
        self.xml = open(os.path.join(os.path.dirname(oca.__file__),
                                     'tests/fixtures/vnet.xml')).read()

    def test_allocate(self):
        self.client.call = Mock(return_value=2)
        assert oca.VirtualNetwork.allocate(self.client, VN_TEMPLATE) == 2

    def test_publish(self):
        self.client.call = Mock(return_value='')
        h = oca.VirtualNetwork(self.xml, self.client)
        h._convert_types()
        h.publish()
        self.client.call.assert_called_once_with('vn.publish', 3, True)

    def test_unpublish(self):
        self.client.call = Mock(return_value='')
        h = oca.VirtualNetwork(self.xml, self.client)
        h._convert_types()
        h.unpublish()
        self.client.call.assert_called_once_with('vn.publish', 3, False)

    def test_repr(self):
        h = oca.VirtualNetwork(self.xml, self.client)
        h._convert_types()
        assert h.__repr__() == '<oca.VirtualNetwork("Red LAN")>'

    def test_chown(self):
        self.client.call = Mock(return_value='')
        h = oca.VirtualNetwork(self.xml, self.client)
        h._convert_types()
        h.chown(2, 2)
        self.client.call.assert_called_once_with('vn.chown', 3, 2, 2)

    def test_address_ranges(self):
        h = oca.VirtualNetwork(self.xml, self.client)
        h._convert_types()
        assert(2==len(h.address_ranges))
        assert(1==h.address_ranges[1].id)
        assert(0==h.address_ranges[0].id)
        assert(" 0 68719479930 1 68719545020"==h.address_ranges[0].allocated)
        assert("10.1.0.10"==h.address_ranges[0].ip)
        assert("00:22:44:66:88:aa"==h.address_ranges[0].mac)
        assert(507==h.address_ranges[0].size)
        assert("IP4"==h.address_ranges[0].type)



# -*- coding: UTF-8 -*-
import os

from mock import Mock

import oca


class TestVmTemplate:
    def setUp(self):
        self.client = oca.Client('test:test')
        self.xml = open(os.path.join(os.path.dirname(oca.__file__),
                                     'tests/fixtures/template.xml')).read()

    def test_allocate(self):
        self.client.call = Mock(return_value=3)
        assert oca.VmTemplate.allocate(self.client, 'name=a') == 3

    def test_delete(self):
        self.client.call = Mock()
        template = oca.VmTemplate(self.xml, self.client)
        template.delete()
        self.client.call.assert_called_once_with('template.delete', '1')

    def test_update(self):
        self.client.call = Mock()
        template = oca.VmTemplate(self.xml, self.client)
        template.update('name=b')
        self.client.call.assert_called_once_with('template.update',
                                                            '1', 'name=b')

    def test_publish(self):
        self.client.call = Mock()
        template = oca.VmTemplate(self.xml, self.client)
        template.publish()
        self.client.call.assert_called_once_with('template.publish',
                                                            '1', True)

    def test_unpublish(self):
        self.client.call = Mock()
        template = oca.VmTemplate(self.xml, self.client)
        template.unpublish()
        self.client.call.assert_called_once_with('template.publish',
                                                            '1', False)

    def test_chown(self):
        self.client.call = Mock()
        template = oca.VmTemplate(self.xml, self.client)
        template.chown(2, 3)
        self.client.call.assert_called_once_with('template.chown',
                                                            '1', 2, 3)

    def test_instantiate_with_default_name(self):
        self.client.call = Mock()
        template = oca.VmTemplate(self.xml, self.client)
        template.instantiate()
        self.client.call.assert_called_once_with('template.instantiate',
                                                            '1', '')

    def test_instantiate_with_custom_name(self):
        self.client.call = Mock()
        template = oca.VmTemplate(self.xml, self.client)
        template.instantiate('asd')
        self.client.call.assert_called_once_with('template.instantiate',
                                                            '1', 'asd')

    def test_repr(self):
        self.client.call = Mock()
        template = oca.VmTemplate(self.xml, self.client)
        assert repr(template) == '<oca.VmTemplate("test1")>'



# -*- coding: UTF-8 -*-
import os
import socket

from mock import Mock
from nose.tools import raises

import oca


class TestClient:
    def setUp(self):
        try:
            del os.environ["ONE_AUTH"]
        except KeyError:
            pass

    def test_secret(self):
        c = oca.Client('test:test')
        assert c.one_auth == 'test:test'

    def test_one_auth(self):
        os.environ["ONE_AUTH"] = os.path.join(os.path.dirname(oca.__file__),
                                             'tests/fixtures/one_auth')
        try:
            c = oca.Client()
            secret = 'test:test'
            assert c.one_auth == secret
        finally:
            os.environ["ONE_AUTH"] = ''

    def test_default_user_path(self):
        os.environ["ONE_AUTH"] = os.path.join(os.path.dirname(oca.__file__),
                                             'tests/fixtures/one_auth')
        c = oca.Client()
        assert c.one_auth == 'test:test'

    @raises(oca.OpenNebulaException)
    def test_wrong_default_user_path(self):
        oca.Client.DEFAULT_ONE_AUTH = '/ad/ads/a/das/d/sad/sad/sa/d/one_auth'
        c = oca.Client()

    @raises(oca.OpenNebulaException)
    def test_invalid_secret(self):
        os.environ["ONE_AUTH"] = os.path.join(os.path.dirname(oca.__file__),
                                             'tests/fixtures/one_auth')
        c = oca.Client('testtest')

    def test_addres(self):
        c = oca.Client('test:test',  "http://8.8.8.8:2633/RPC2")
        assert c.one_address == "http://8.8.8.8:2633/RPC2"

    def test_one_xml_rpc(self):
        os.environ["ONE_AUTH"] = os.path.join(os.path.dirname(oca.__file__),
                                             'tests/fixtures/one_auth')
        os.environ["ONE_XMLRPC"] = "http://8.8.8.8:2633/RPC2"
        try:
            c = oca.Client()
            assert c.one_address == "http://8.8.8.8:2633/RPC2"
        finally:
            os.environ["ONE_XMLRPC"] = ''

    def test_defaul_xmlrpc(self):
        c = oca.Client('test:test')
        assert c.one_address == oca.Client.DEFAULT_ONE_ADDRESS

    def test_return_two_values_call(self):
        c = oca.Client('test:test')
        c.server.one = Mock()
        c.server.one.test_method = lambda x: [True, '2', 0]
        assert c.call('test_method') == '2'

    def test_return_one_value_call(self):
        c = oca.Client('test:test')
        c.server.one = Mock()
        c.server.one.test_method = lambda x: [True, '', 0]
        assert c.call('test_method') == ''

    @raises(oca.OpenNebulaException)
    def test_retrurn_error_code_0_call(self):
        c = oca.Client('test:test')
        c.server.one = Mock()
        c.server.one.test_method = lambda x: [False, '2', 1]
        c.call('test_method')


    @raises(oca.OpenNebulaException)
    def test_invalid_call(self):
        c = oca.Client('test:test')
        c.server.one = Mock()
        c.server.one.test_method = lambda x: [False, '2']
        c.call('test_method')

    @raises(socket.error)
    def test_connection_error(self):
        c = oca.Client('test:test')
        c.server.one = Mock()
        c.server.one.test_method = Mock(side_effect=socket.error(1))
        c.call('test_method')




# -*- coding: UTF-8 -*-
import os

from mock import Mock

import oca
import oca.pool


IMAGE_TEMPLATE = '''NAME          = "Ubuntu Desktop"
PATH          = /home/cloud/images/ubuntu-desktop/disk.0
PUBLIC        = YES
DESCRIPTION   = "Ubuntu 10.04 desktop for students."'''


class TestImage:
    def setUp(self):
        self.client = oca.Client('test:test')
        self.xml = open(os.path.join(os.path.dirname(oca.__file__),
                                         'tests/fixtures/image.xml')).read()

    def test_allocate(self):
        self.client.call = Mock(return_value=2)
        assert oca.Image.allocate(self.client, IMAGE_TEMPLATE) == 2

    def test_enable(self):
        self.client.call = Mock(return_value='')
        h = oca.Image(self.xml, self.client)
        h.enable()
        self.client.call.assert_called_once_with('image.enable', '1', True)

    def test_disable(self):
        self.client.call = Mock(return_value='')
        h = oca.Image(self.xml, self.client)
        h.disable()
        self.client.call.assert_called_once_with('image.enable', '1', False)

    def test_update(self):
        self.client.call = Mock(return_value='')
        h = oca.Image(self.xml, self.client)
        new_content = 'DEV_PREFIX=hd\nNAME=Debian\nTYPE=OS'
        h.update(new_content)
        self.client.call.assert_called_once_with('image.update', '1',
                                                    new_content)

    def test_publish(self):
        self.client.call = Mock(return_value='')
        h = oca.Image(self.xml, self.client)
        assert h.publish() is None

    def test_unpublish(self):
        self.client.call = Mock(return_value='')
        h = oca.Image(self.xml, self.client)
        assert h.unpublish() is None

    def test_set_persistent(self):
        self.client.call = Mock(return_value='')
        h = oca.Image(self.xml, self.client)
        h.set_persistent()
        self.client.call.assert_called_once_with('image.persistent',
                                                '1', True)

    def test_set_nonpersistent(self):
        self.client.call = Mock(return_value='')
        h = oca.Image(self.xml, self.client)
        h.set_nonpersistent()
        self.client.call.assert_called_once_with('image.persistent',
                                                '1', False)

    def test_states(self):
        for i in range(len(oca.Image.IMAGE_STATES)):
            h = oca.Image('<IMAGE><ID>2</ID><STATE>%s</STATE></IMAGE>' % i,
                      self.client)
            assert h.str_state == oca.Image.IMAGE_STATES[i]
            short_image_state = oca.Image.SHORT_IMAGE_STATES[oca.Image.IMAGE_STATES[i]]
            assert h.short_state == short_image_state

    def test_repr(self):
        h = oca.Image(self.xml, self.client)
        assert h.__repr__() == '<oca.Image("MATLAB install CD")>'

    def test_types(self):
        for i in range(len(oca.Image.IMAGE_TYPES)):
            h = oca.Image('<IMAGE><ID>2</ID><TYPE>%s</TYPE></IMAGE>' % i,
                    self.client)
            assert h.str_type == oca.Image.IMAGE_TYPES[i]
            short_image_type = oca.Image.SHORT_IMAGE_TYPES[oca.Image.IMAGE_TYPES[i]]
            assert h.short_type == short_image_type

    def test_template(self):
        i = oca.Image(self.xml, self.client)
        i._convert_types()
        assert isinstance(i.template, oca.pool.Template)

    def test_chown(self):
        self.client.call = Mock(return_value='')
        h = oca.Image(self.xml, self.client)
        h.chown(10, 10)
        self.client.call.assert_called_once_with('image.chown',
                                                '1', 10, 10)



# -*- coding: UTF-8 -*-
from pool import Pool, PoolElement, Template, extractString

class Datastore(PoolElement):
    METHODS = {
        #'info'     : 'datastore.info',
        'allocate' : 'datastore.allocate',
        'delete'   : 'datastore.delete',
        #'enable'   : 'datastore.enable',
        #'update'   : 'datastore.update'
    }

    XML_TYPES = {
        'id'            : int,
        'name'          : extractString,
        'uid'           : int,
        'gid'           : int,
        'uname'         : extractString,
        'gname'         : extractString,
        #'permissions'   : Permissions,
        'ds_mad'        : extractString,
        'tm_mad'        : extractString,
        'base_path'     : extractString,
        'type'          : int,
        'disk_type'     : int,
        #'state'         : ???,
        'cluster_id'    : int,
        'cluster'       : extractString,
        'total_mb'      : int,
        'free_mb'       : int,
        'used_mb'       : int,
        'image_ids'     : ['IMAGES', lambda images: map(lambda image_id: int(image_id.text), images)],
        'template'      : ['TEMPLATE', Template],
    }

    ELEMENT_NAME = 'DATASTORE'

    @staticmethod
    def allocate(client, datastore_template):
        '''
        Adds a datastore to the datastore list

        Arguments

        ``datastore_template``
           Template for the datastore to add
        '''
        datastore_id = client.call(Datastore.METHODS['allocate'], datastore_template)
        return datastore_id

    def __init__(self, xml, client):
        super(Datastore, self).__init__(xml, client)
        self._convert_types()

    def __repr__(self):
        return '<oca.Datastore("%s")>' % self.name


class DatastorePool(Pool):
    METHODS = {
            'info' : 'datastorepool.info',
    }

    def __init__(self, client):
        super(DatastorePool, self).__init__('DATASTORE_POOL', 'DATASTORE', client)

    def _factory(self, xml):
        c = Datastore(xml, self.client)
        return c



# -*- coding: UTF-8 -*-
import os

from mock import Mock
from nose.tools import raises

import oca
import oca.pool


class TestVirtualMachinePool:
    def setUp(self):
        self.client = oca.Client('test:test')
        self.xml = open(os.path.join(os.path.dirname(oca.__file__),
                                     'tests/fixtures/vmpool.xml')).read()

    def test_vm_pool_info(self):
        self.client.call = Mock(return_value=self.xml)
        pool = oca.VirtualMachinePool(self.client)
        pool.info()
        assert len(list(pool)) == 3

    def test_iterate_before_info(self):
        pool = oca.VirtualMachinePool(self.client)
        assert list(pool) == []

    def test_get_by_id(self):
        self.client.call = Mock(return_value=self.xml)
        pool = oca.VirtualMachinePool(self.client)
        pool.info()
        assert  pool.get_by_id(8).id == 8

    def test_get_by_name(self):
        self.client.call = Mock(return_value=self.xml)
        pool = oca.VirtualMachinePool(self.client)
        pool.info()
        assert  pool.get_by_name('vm-in').name == 'vm-in'

    @raises(oca.pool.WrongIdError)
    def test_wrong_get_by_id(self):
        self.client.call = Mock(return_value=self.xml)
        pool = oca.VirtualMachinePool(self.client)
        pool.info()
        pool.get_by_id(1010011010)

    @raises(oca.pool.WrongNameError)
    def test_wrong_get_by_name(self):
        self.client.call = Mock(return_value=self.xml)
        pool = oca.VirtualMachinePool(self.client)
        pool.info()
        pool.get_by_name('wrong-vm-name')



# -*- coding: UTF-8 -*-
import os

from mock import Mock

import oca


class TestImagePool:
    def setUp(self):
        self.client = oca.Client('test:test')
        self.xml = open(os.path.join(os.path.dirname(oca.__file__),
                                     'tests/fixtures/imagepool.xml')).read()

    def test_info(self):
        self.xml = self.xml.replace('\n', '')
        self.xml = self.xml.replace(' ', '')
        self.client.call = Mock(return_value=self.xml)
        pool = oca.ImagePool(self.client)
        pool.info()
        assert len(list(pool)) == 3




# -*- coding: UTF-8 -*-
from pool import Pool, PoolElement, Template, extractString


class HostShare(Template):
    def __repr__(self):
        return '<oca.vm.HostShare()>'


class Host(PoolElement):
    METHODS = {
        'info'     : 'host.info',
        'allocate' : 'host.allocate',
        'delete'   : 'host.delete',
        'enable'   : 'host.enable',
        'update'   : 'host.update'
    }

    INIT = 0
    MONITORING_MONITORED = 1 # Currently monitoring, previously MONITORED
    MONITORED = 2
    ERROR = 3
    DISABLED = 4
    MONITORING_ERROR = 5     # Currently monitoring, previously ERROR
    MONITORING_INIT = 6      # Currently monitoring, previously initialized
    MONITORING_DISABLED = 7  # Currently monitoring, previously DISABLED
    HOST_STATES = ['INIT', 'MONITORING_MONITORED', 'MONITORED', 'ERROR', 'DISABLED',
                   'MONITORING_ERROR', 'MONITORING_INIT', 'MONITORING_DISABLED']

    SHORT_HOST_STATES = {
        'INIT':                 'on',
        'MONITORING_MONITORED': 'on',
        'MONITORED':            'on',
        'ERROR':                'err',
        'DISABLED':             'off',
        'MONITORING_ERROR':     'on',
        'MONITORING_INIT':      'on',
        'MONITORING_DISABLED':  'on',
    }

    XML_TYPES = {
        'id':             int,
        'name':           extractString,
        'state':          int,
        'im_mad':         extractString,
        'vm_mad':         extractString,
        'vn_mad':         extractString,
        'last_mon_time':  int,
        'cluster':        extractString,
        'cluster_id':     int,
        'vm_ids':         ['VMS', lambda vms: map(lambda vmid: int(vmid.text), vms)],
        'template':       ['TEMPLATE', Template],
        'host_share':     ['HOST_SHARE', HostShare],
    }

    ELEMENT_NAME = 'HOST'

    @staticmethod
    def allocate(client, hostname, im, vmm, tm, cluster_id=-1):
        '''
        Adds a host to the host list

        Arguments

        ``hostname``
           Hostname machine to add

        ``im``
           Information manager'

        ``vmm``
           Virtual machine manager.

        ``tm``
           Transfer manager
        '''
        host_id = client.call(Host.METHODS['allocate'], hostname, im, vmm, tm, cluster_id)
        return host_id

    def __init__(self, xml, client):
        super(Host, self).__init__(xml, client)
        self.id = self['ID'] if self['ID'] else None

    def enable(self):
        '''
        Enable this host
        '''
        self.client.call(self.METHODS['enable'], self.id, True)

    def disable(self):
        '''
        Disable this host.
        '''
        self.client.call(self.METHODS['enable'], self.id, False)

    def update(self, template, merge=False):
        '''
        Update the template of this host. If merge is false (default),
        the existing template is replaced.
        '''
        self.client.call(self.METHODS['update'], self.id, template, 1 if merge else 0)

    @property
    def str_state(self):
        '''
        String representation of host state.
        One of 'INIT', 'MONITORING', 'MONITORED', 'ERROR', 'DISABLED'
        '''
        return self.HOST_STATES[int(self.state)]

    @property
    def short_state(self):
        '''
        Short string representation of host state. One of 'on', 'off', 'err'
        '''
        return self.SHORT_HOST_STATES[self.str_state]

    def __repr__(self):
        return '<oca.Host("%s")>' % self.name


class HostPool(Pool):
    METHODS = {
            'info' : 'hostpool.info',
    }

    def __init__(self, client):
        super(HostPool, self).__init__('HOST_POOL', 'HOST', client)

    def _factory(self, xml):
        h = Host(xml, self.client)
        h._convert_types()
        return h




# -*- coding: UTF-8 -*-
from pool import Pool, PoolElement, Template, extractString

class Cluster(PoolElement):
    METHODS = {
        #'info'     : 'cluster.info',
        'allocate' : 'cluster.allocate',
        'delete'   : 'cluster.delete',
        #'enable'   : 'cluster.enable',
        #'update'   : 'cluster.update'
    }

    XML_TYPES = {
        'id'            : int,
        'name'          : extractString,
        'host_ids'      : ['HOSTS', lambda hosts: map(lambda host_id: int(host_id.text), hosts)],
        'datastore_ids' : ['DATASTORES', lambda datastores: map(lambda datastore_id: int(datastore_id.text), datastores)],
        'vnet_ids'      : ['VNETS', lambda vnets: map(lambda vnet_id: int(vnet_id.text), vnets)],
        'template'      : ['TEMPLATE', Template],
    }

    ELEMENT_NAME = 'CLUSTER'

    @staticmethod
    def allocate(client, cluster_name):
        '''
        Adds a cluster to the cluster list

        Arguments

        ``cluster_name``
           Clustername to add
        '''
        cluster_id = client.call(Cluster.METHODS['allocate'], cluster_name)
        return cluster_id

    def __init__(self, xml, client):
        super(Cluster, self).__init__(xml, client)
        self._convert_types()

    def __repr__(self):
        return '<oca.Cluster("%s")>' % self.name


class ClusterPool(Pool):
    METHODS = {
            'info' : 'clusterpool.info',
    }

    def __init__(self, client):
        super(ClusterPool, self).__init__('CLUSTER_POOL', 'CLUSTER', client)

    def _factory(self, xml):
        c = Cluster(xml, self.client)
        return c




# -*- coding: UTF-8 -*-
from pool import Pool, PoolElement, Template, extractString


class History(Template):
    def __repr__(self):
        return '<oca.vm.History("seq={0}")>'.format(self.seq)


class VirtualMachine(PoolElement):
    METHODS = {
        'info':      'vm.info',
        'allocate':  'vm.allocate',
        'action':    'vm.action',
        'migrate':   'vm.migrate',
        'deploy':    'vm.deploy',
        'savedisk':  'vm.savedisk',
        'delete':    'vm.delete',
        'chown':     'vm.chown',
        'update':    'vm.update',
    }

    INIT = 0
    PENDING = 1
    HOLD = 2
    ACTIVE = 3
    STOPPED = 4
    SUSPENDED = 5
    DONE = 6
    FAILED = 7
    VM_STATE = ['INIT', 'PENDING', 'HOLD', 'ACTIVE', 'STOPPED',
                'SUSPENDED', 'DONE', 'FAILED']

    SHORT_VM_STATES = {
        'INIT':       'init',
        'PENDING':    'pend',
        'HOLD':       'hold',
        'ACTIVE':     'actv',
        'STOPPED':    'stop',
        'SUSPENDED':  'susp',
        'DONE':       'done',
        'FAILED':     'fail'
    }

    LCM_STATE = ['LCM_INIT', 'PROLOG', 'BOOT', 'RUNNING', 'MIGRATE',
                 'SAVE_STOP', 'SAVE_SUSPEND', 'SAVE_MIGRATE',
                 'PROLOG_MIGRATE', 'PROLOG_RESUME', 'EPILOG_STOP', 'EPILOG',
                 'SHUTDOWN', 'CANCEL', 'FAILURE', 'DELETE', 'UNKNOWN']

    SHORT_LCM_STATES = {
        'LCM_INIT':       'init',
        'PROLOG':         'prol',
        'BOOT':           'boot',
        'RUNNING':        'runn',
        'MIGRATE':        'migr',
        'SAVE_STOP':      'save',
        'SAVE_SUSPEND':   'save',
        'SAVE_MIGRATE':   'save',
        'PROLOG_MIGRATE': 'migr',
        'PROLOG_RESUME':  'prol',
        'EPILOG_STOP':    'epil',
        'EPILOG':         'epil',
        'SHUTDOWN':       'shut',
        'CANCEL':         'shut',
        'FAILURE':        'fail',
        'DELETE':         'dele',
        'UNKNOWN':        'unkn',
    }

    MIGRATE_REASON = ['NONE', 'ERROR', 'STOP_RESUME', 'USER', 'CANCEL']

    SHORT_MIGRATE_REASON = {
        'NONE':           'none',
        'ERROR':          'erro',
        'STOP_RESUME':    'stop',
        'USER':           'user',
        'CANCEL':         'canc'
    }

    XML_TYPES = {
        'id':            int,
        'uid':           int,
        'gid':           int,
        'uname':         extractString,
        'gname':         extractString,
        'name':          extractString,
        #'permissions': ???,
        'last_poll':     int,
        'state':         int,
        'lcm_state':     int,
        'resched':       int,
        'stime':         int,
        'etime':         int,
        'deploy_id':     extractString,
        'template':      ['TEMPLATE', Template, ['NIC', 'DISK']],
        'user_template': ['USER_TEMPLATE', Template],
        'history_records':  ['HISTORY_RECORDS', lambda x: [History(i)
                                        for i in x] if x is not None else []],
    }

    ELEMENT_NAME = 'VM'

    @staticmethod
    def allocate(client, template):
        '''
        allocates a virtual machine description from the given template string

        Arguments

        ``template``
           a string containing the template of the vm
        '''
        vm_id = client.call(VirtualMachine.METHODS['allocate'], template)
        return vm_id

    def __init__(self, xml, client):
        super(VirtualMachine, self).__init__(xml, client)
        self.id = self['ID'] if self['ID'] else None

    def deploy(self, host_id):
        '''
        initiates the instance of the given vmid on the target host

        Arguments

        ``host_id``
           the host id (hid) of the target host where the VM will be
           instantiated.
        '''
        self.client.call(self.METHODS['deploy'], self.id, host_id)

    def migrate(self, dest_host):
        '''
        migrates virtual machine to the target host

        Arguments

        ``dest_host``
           the target host id
        '''
        self.client.call(self.METHODS['migrate'], self.id, dest_host, False)

    def live_migrate(self, dest_host):
        '''
        live migrates virtual machine to the target host

        Arguments

        ``dest_host``
           the target host id
        '''
        self.client.call(self.METHODS['migrate'], self.id, dest_host, True)

    def save_disk(self, disk_id, dest_disk):
        '''
        Sets the disk to be saved in the given image

        Arguments

        ``disk_id``
           disk id of the disk we want to save
        ``dest_disk``
           image id where the disk will be saved.
        '''
        self.client.call(self.METHODS['savedisk'], self.id, disk_id, dest_disk)

    def shutdown(self):
        '''
        Shutdowns an already deployed VM
        '''
        self._action('shutdown')

    def cancel(self):
        '''
        Cancels a running VM
        '''
        self._action('cancel')

    def hold(self):
        '''
        Sets a VM to hold state, scheduler will not deploy it
        '''
        self._action('hold')

    def release(self):
        '''
        Releases a VM from hold state
        '''
        self._action('release')

    def stop(self):
        '''
        Stops a running VM
        '''
        self._action('stop')

    def suspend(self):
        '''
        Saves a running VM
        '''
        self._action('suspend')

    def resume(self):
        '''
        Resumes the execution of a saved VM
        '''
        self._action('resume')

    def finalize(self):
        '''
        Deletes a VM from the pool and DB
        '''
        self._action('finalize')

    def restart(self):
        '''
        Resubmits the VM after failure
        '''
        self._action('restart')

    def resubmit(self):
        '''
        Redeploy the VM.
        '''
        self._action('resubmit')

    def delete(self):
        '''
        Delete the VM.
        '''
        self._action('delete')

    def _action(self, action):
        self.client.call(self.METHODS['action'], action, self.id)

    def __repr__(self):
        return '<oca.VirtualMachine("%s")>' % self.name

    @property
    def str_state(self):
        '''
        String representation of virtual machine state.
        One of: INIT, PENDING, HOLD, ACTIVE, STOPPED, SUSPENDED,
        DONE, FAILED
        '''
        return self.VM_STATE[int(self.state)]

    @property
    def short_state(self):
        '''
        Short string representation of virtual machine state.
        One of: init, pend, hold, actv, stop, susp, done, fail
        '''
        return self.SHORT_VM_STATES[self.str_state]

    @property
    def str_lcm_state(self):
        '''
        String representation of virtual machine LCM state.
        One of: LCM_INIT, PROLOG, BOOT, RUNNING, MIGRATE,
        SAVE_STOP, SAVE_SUSPEND, SAVE_MIGRATE, PROLOG_MIGRATE,
        PROLOG_RESUME, EPILOG_STOP, EPILOG, SHUTDOWN, CANCEL,
        FAILURE, DELETE, UNKNOWN
        '''
        return self.LCM_STATE[int(self.lcm_state)]

    @property
    def short_lcm_state(self):
        '''
        Short string representation of virtual machine LCM state.
        One of: init, prol, boot, runn, migr, save, save,
        save, migr, prol, epil, shut, shut, fail,
        dele, unkn
        '''
        return self.SHORT_LCM_STATES[self.str_lcm_state]

    def update(self, template, merge=False):
        '''
        Update the template of this host. If merge is false (default),
        the existing template is replaced.
        '''
        self.client.call(self.METHODS['update'], self.id, template, 1 if merge else 0)


class VirtualMachinePool(Pool):
    METHODS = {
            'info' : 'vmpool.info',
    }

    def __init__(self, client):
        super(VirtualMachinePool, self).__init__('VM_POOL', 'VM', client)

    def _factory(self, xml):
        vm = VirtualMachine(xml, self.client)
        vm._convert_types()
        return vm

    def info(self, filter=-3, range_start=-1, range_end=-1, vm_state=-1):
        '''
        Retrives/Refreshes virtual machine pool information

        ``filter``
            Filter flag. By defaults retrives only connected user reources.

        ``range_start``
            Range start ID. -1 for all

        ``range_end``
            Range end ID. -1 for all

        ``vm_state``

            VM state to filter by.

            * \-2	 Any state, including DONE
            * \-1	 Any state, except DONE (Defualt)
            * 0	 INIT
            * 1	 PENDING
            * 2	 HOLD
            * 3	 ACTIVE
            * 4	 STOPPED
            * 5	 SUSPENDED
            * 6	 DONE
            * 7	 FAILED

        '''
        super(VirtualMachinePool, self).info(filter, range_start,
                                             range_end, vm_state)



# -*- coding: UTF-8 -*-
import xmlrpclib
import os
import hashlib
import re
import socket
import httplib

from host import Host, HostPool
from vm import VirtualMachine, VirtualMachinePool
from user import User, UserPool
from image import Image, ImagePool
from vn import VirtualNetwork, VirtualNetworkPool
from group import Group, GroupPool
from template import VmTemplate, VmTemplatePool
from exceptions import OpenNebulaException
from cluster import Cluster, ClusterPool
from datastore import Datastore, DatastorePool


CONNECTED = -3
ALL = -2
CONNECTED_AND_GROUP = -1

class TimeoutHTTPConnection(httplib.HTTPConnection):
    def connect(self):
        httplib.HTTPConnection.connect(self)
        self.sock.settimeout(self.timeout)

class TimeoutHTTP(httplib.HTTP):
    _connection_class = TimeoutHTTPConnection

    def set_timeout(self, timeout):
        self._conn.timeout = timeout

class ProxiedTransport(xmlrpclib.Transport):
    def __init__(self, timeout=socket._GLOBAL_DEFAULT_TIMEOUT, *args, **kwargs):
        xmlrpclib.Transport.__init__(self, *args, **kwargs)
        self.timeout = timeout
    def set_proxy(self, proxy):
        self.proxy = proxy
    def make_connection(self, host):
        self.realhost = host
        h = httplib.HTTPConnection(self.proxy)
        return h
    def send_request(self, connection, handler, request_body):
        connection.putrequest("POST", 'http://%s%s' % (self.realhost, handler))
    def send_host(self, connection, host):
        connection.putheader('Host', self.realhost)

class Client(object):
    '''
    The client class, represents the connection with the core and handles the
    xml-rpc calls(see http://www.opennebula.org/documentation:rel3.2:api)
    '''
    DEFAULT_ONE_AUTH = "~/.one/one_auth"
    ONE_AUTH_RE = re.compile('^(.+?):(.+)$')
    DEFAULT_ONE_ADDRESS = "http://localhost:2633/RPC2"

    def __init__(self, secret=None, address=None, proxy=None):
        if secret:
            one_secret = secret
        elif "ONE_AUTH" in os.environ and os.environ["ONE_AUTH"]:
            one_auth = os.path.expanduser(os.environ["ONE_AUTH"])
            with open(one_auth) as f:
                one_secret = f.read().strip()
        elif os.path.exists(os.path.expanduser(self.DEFAULT_ONE_AUTH)):
            with open(os.path.expanduser(self.DEFAULT_ONE_AUTH)) as f:
                one_secret = f.read().strip()
        else:
            raise OpenNebulaException('ONE_AUTH file not present')

        one_secret = self.ONE_AUTH_RE.match(one_secret)
        if one_secret:
            user = one_secret.groups()[0]
            password = one_secret.groups()[1]
        else:
            raise OpenNebulaException("Authorization file malformed")

        self.one_auth = '{0}:{1}'.format(user, password)

        if address:
            self.one_address = address
        elif "ONE_XMLRPC" in os.environ and os.environ["ONE_XMLRPC"]:
            self.one_address = os.environ["ONE_XMLRPC"]
        else:
            self.one_address = self.DEFAULT_ONE_ADDRESS

        if proxy:
            p = ProxiedTransport(timeout=100)
            p.set_proxy(proxy)
            self.server = xmlrpclib.ServerProxy(self.one_address, transport=p)
        else:
            self.server = xmlrpclib.ServerProxy(self.one_address)
        

    def call(self, function, *args):
        '''
        Calls rpc function.

        Arguments

        ``function``
           OpenNebula xmlrpc funtcion name (without preceding 'one.')

        ``args``
           function arguments

        '''
        try:
            func = getattr(self.server.one, function)
            ret = func(self.one_auth, *args)
            try:
                is_success, data, return_code = ret
            except ValueError:
                data = ''
                is_success = False
        except socket.error, e:
            #connection error
            raise e
        if not is_success:
            raise OpenNebulaException(data)
        return data

    def version(self):
        '''
        Get the version of the connected OpenNebula server.
        '''
        return self.call('system.version')

__all__ = [Client, OpenNebulaException, Host, HostPool, VirtualMachine,
        VirtualMachinePool, User, UserPool,
        Image, ImagePool, VirtualNetwork, VirtualNetworkPool,
        Group, GroupPool, VmTemplate, VmTemplatePool, ALL, CONNECTED,
        Cluster, ClusterPool, Datastore, DatastorePool]




# -*- coding: UTF-8 -*-
import os

from mock import Mock

import oca


class TestHost:
    def setUp(self):
        self.client = oca.Client('test:test')
        self.xml = open(os.path.join(os.path.dirname(oca.__file__),
                                         'tests/fixtures/host.xml')).read()

    def test_allocate(self):
        self.client.call = Mock(return_value=7)
        host_id = oca.Host.allocate(self.client, 'host', 'im_xen',
                               'vmm_xen', 'tm_nfs')
        assert host_id == 7

    def test_enable(self):
        self.client.call = Mock(return_value='')
        h = oca.Host(self.xml, self.client)
        h.enable()
        self.client.call.assert_called_once_with('host.enable', '7', True)

    def test_disable(self):
        self.client.call = Mock(return_value='')
        h = oca.Host(self.xml, self.client)
        h.disable()
        self.client.call.assert_called_once_with('host.enable', '7', False)

    def test_states(self):
        for i in range(len(oca.Host.HOST_STATES)):
            h = oca.Host('<HOST><ID>2</ID><STATE>{0}</STATE></HOST>'.format(i),
                     self.client)
            assert h.str_state == oca.Host.HOST_STATES[i]
            assert h.short_state == oca.Host.SHORT_HOST_STATES[oca.Host.HOST_STATES[i]]

    def test_repr(self):
        h = oca.Host(self.xml, self.client)
        assert h.__repr__() == '<oca.Host("dummyhost")>'

    def test_host_share_repr(self):
        h = oca.Host(self.xml, self.client)
        h._convert_types()
        share = h.host_share
        assert repr(share) == '<oca.vm.HostShare()>'

    def test_host_vm_ids(self):
        h = oca.Host(self.xml, self.client)
        h._convert_types()
        vm_ids = h.vm_ids
        assert vm_ids == [82,84,85,95,96]



# -*- coding: UTF-8 -*-
import os

from mock import Mock

import oca


class TestUser:
    def setUp(self):
        self.client = oca.Client('test:test')
        self.xml = open(os.path.join(os.path.dirname(oca.__file__),
                                     'tests/fixtures/user.xml')).read()

    def test_allocate(self):
        self.client.call = Mock(return_value=1)
        assert oca.User.allocate(self.client, 'jon', 'secret') == 1

    def test_repr(self):
        u = oca.User(self.xml, self.client)
        assert u.__repr__() == '<oca.User("dan")>'

    def test_change_passwd(self):
        self.client.call = Mock(return_value='')
        u = oca.User(self.xml, self.client)
        u.change_passwd('secret2')
        self.client.call.assert_called_once_with('user.passwd', '3', 'secret2')

    def test_delete(self):
        self.client.call = Mock(return_value='')
        vm = oca.User(self.xml, self.client)
        vm.delete()
        self.client.call.assert_called_once_with('user.delete', '3')

    def test_change_group(self):
        self.client.call = Mock(return_value='')
        vm = oca.User(self.xml, self.client)
        vm.chgrp(3)
        self.client.call.assert_called_once_with('user.chgrp', '3', 3)




# -*- coding: UTF-8 -*-
import os

from mock import Mock

import oca


class TestGroup:
    def setUp(self):
        self.client = oca.Client('test:test')
        self.xml = open(os.path.join(os.path.dirname(oca.__file__),
                                     'tests/fixtures/group.xml')).read()

    def test_allocate(self):
        self.client.call = Mock(return_value=3)
        assert oca.Group.allocate(self.client, 'test') == 3

    def test_delete(self):
        self.client.call = Mock()
        group = oca.Group(self.xml, self.client)
        group.delete()
        self.client.call.assert_called_once_with('group.delete', '1')

    def test_repr(self):
        self.client.call = Mock()
        group = oca.Group(self.xml, self.client)
        assert repr(group) == '<oca.Group("users")>'

    def test_convert_types(self):
        group = oca.Group(self.xml, None)
        group._convert_types()
        assert group.id == 1
        assert group.name == "users"
        assert group.template.hello == "world"
        assert group.users == [1,2]




# vim:  set fileencoding=utf8
# encoding:utf8
#
# Author: Matthias Schmitz <matthias@sigxcpu.org>

import unittest
import os
import oca


@unittest.skipUnless(os.environ.has_key('OCA_INT_TESTS'),
                     "Skipping integration tests")
class IntTestClient(unittest.TestCase):
    def setUp(self):
        try:
            del os.environ["ONE_AUTH"]
        except KeyError:
            pass

        self.c = oca.Client(os.environ['OCA_INT_TESTS_ONE_AUTH'],
                            os.environ['OCA_INT_TESTS_ONE_XMLRPC'])

    def test_connection(self):
        self.assertIn(os.environ['OCA_INT_TESTS'], self.c.version())



# -*- coding: UTF-8 -*-
from pool import XMLElement, Pool, PoolElement, Template


class AddressRange(XMLElement):
    XML_TYPES = {
            'id'   : ["AR_ID", lambda xml: int(xml.text)],
            'size' : int,
            #'template' : ['TEMPLATE', Template],
    }

    def __init__(self, xml):
        super(AddressRange, self).__init__(xml)
        self._convert_types()
        self.id = self['AR_ID'] if self['AR_ID'] else None

class AddressRangeList(list):
    def __init__(self, xml):
        self.xml = xml
        for element in self.xml:
            self.append(self._factory(element))

    def _factory(self, xml):
        v = AddressRange(xml)
        v._convert_types()
        return v

class VirtualNetwork(PoolElement):
    METHODS = {
        'info'     : 'vn.info',
        'allocate' : 'vn.allocate',
        'delete'   : 'vn.delete',
        'publish'  : 'vn.publish',
        'chown'    : 'vn.chown',
    }

    XML_TYPES = {
            'id'       : int,
            'uid'      : int,
            'gid'      : int,
            'uname'    : str,
            'gname'    : str,
            'name'     : str,
            #'type'     : int,
            'bridge'   : str,
            #'public'   : bool,
            'used_leases' : int,
            'template' : ['TEMPLATE', Template],
            'address_ranges': ['AR_POOL', AddressRangeList],
    }

    ELEMENT_NAME = 'VNET'

    @staticmethod
    def allocate(client, template):
        '''
        allocates a new virtual network in OpenNebula

        Arguments

        ``template``
           a string containing the template of the virtual network
        '''
        vn_id = client.call(VirtualNetwork.METHODS['allocate'], template)
        return vn_id

    def __init__(self, xml, client):
        super(VirtualNetwork, self).__init__(xml, client)
        self.id = self['ID'] if self['ID'] else None

    def publish(self):
        '''
        Publishes a virtual network.
        '''
        self.client.call(self.METHODS['publish'], self.id, True)

    def unpublish(self):
        '''
        Unpublishes a virtual network.
        '''
        self.client.call(self.METHODS['publish'], self.id, False)

    def chown(self, uid, gid):
        '''
        Changes the owner/group

        Arguments

        ``uid``
        New owner id. Set to -1 to leave current value
        ``gid``
        New group id. Set to -1 to leave current value
        '''
        self.client.call(self.METHODS['chown'], self.id, uid, gid)

    def __repr__(self):
        return '<oca.VirtualNetwork("%s")>' % self.name

class VirtualNetworkPool(Pool):
    METHODS = {
            'info' : 'vnpool.info',
    }

    def __init__(self, client):
        super(VirtualNetworkPool, self).__init__('VNET_POOL', 'VNET', client)

    def _factory(self, xml):
        v = VirtualNetwork(xml, self.client)
        v._convert_types()
        return v



import oca

class TestIntegration:
    #def setUp(self):
    #    self.client = oca.Client("oneadmin:one", "http://localhost:2633/RPC2")
    #    cp = oca.ClusterPool(self.client)
    #    cp.info()
    #    for cluster in cp:
    #        cluster.delete()

    #def test_integration(self):
    #    self.cluster_id = oca.Cluster.allocate(self.client, "IT-Cluster")
    #    #self.datastore_id = oca.
    pass



# -*- coding: UTF-8 -*-
import xml.etree.ElementTree as ET
import new

from exceptions import OpenNebulaException


class WrongNameError(OpenNebulaException):
    pass


class WrongIdError(OpenNebulaException):
    pass

def extractString(xml_or_string):
    if isinstance(xml_or_string, str):
        return xml_or_string
    else:
        return xml_or_string.text or ''

class Template(object):
    def __init__(self, xml_element, multiple=[]):
        self.xml = ET.tostring(xml_element)
        self.xml_element = xml_element
        self.multiple = multiple
        self.parse()

    def parse(self):
        for element in self.xml_element:
            tag = element.tag
            if tag in self.multiple:
                self.parse_multiple(tag, element)
            else:
                setattr(self, tag.lower(), element.text)

    def parse_multiple(self, tag, element):
        attr = tag.lower() + 's'
        attr_list = getattr(self, attr, [])

        class_obj = new.classobj(tag.capitalize(), (Template,), {})

        attr_list.append(class_obj(element))
        setattr(self, attr, attr_list)


class XMLElement(object):
    XML_TYPES = {}

    def __init__(self, xml=None):
        if not (xml is None or ET.iselement(xml)):
            xml = ET.fromstring(xml)
        self.xml = xml

    def _initialize_xml(self, xml, root_element):
        self.xml = ET.fromstring(xml)
        if self.xml.tag != root_element.upper():
            self.xml = None
        self._convert_types()

    def __getitem__(self, key):
        value = self.xml.find(key.upper())
        if value is not None:
            if value.text:
                return value.text
            else:
                return value
        else:
            raise IndexError("Key {0} not found!".format(key))

    def __getattr__(self, name):
        try:
            return self[name]
        except (IndexError, TypeError):
            raise AttributeError(name)

    def _convert_types(self):
        for name, fun in self.XML_TYPES.items():
            if isinstance(fun, list):
                tag, cls = fun[0], fun[1]
                xml = self.xml.find(tag)
                setattr(self, name, cls(xml, *fun[2:]))
            else:
                setattr(self, name, fun(self[name]))


class Pool(list, XMLElement):
    def __init__(self, pool, element, client):
        super(Pool, self).__init__()

        self.pool_name = pool
        self.element_name = element
        self.client = client

    def info(self, filter=-3, range_start=-1, range_end=-1, *args):
        '''
        Retrives/Refreshes resource pool information

        ``filter``
            Filter flag. By defaults retrives only connected user reources.

        ``range_start``
            Range start ID. -1 for all

        ``range_end``
            Range end ID. -1 for all
        '''
        self[:] = []
        data = self.client.call(self.METHODS['info'], filter,
                                        range_start, range_end, *args)
        self._initialize_xml(data, self.pool_name)
        for element in self.xml.findall(self.element_name):
            self.append(self._factory(element))

    def _factory(self):
        pass

    def get_by_id(self, id):
        for i in self:
            if i.id == id:
                return i
        raise WrongIdError()

    def get_by_name(self, name):
        for i in self:
            if i.name == name:
                return i
        raise WrongNameError()


class PoolElement(XMLElement):
    def __init__(self, xml, client):
        super(PoolElement, self).__init__(xml)
        self.client = client

    @classmethod
    def new_with_id(cls, client, element_id):
        '''
        Retrives object which id equals ```id```.

        Arguments

        ```client```
           oca.Client object.
        ```element_id``
           object id.
        '''
        element = cls.ELEMENT_NAME
        xml = '<{0}><ID>{1}</ID></{0}>'.format(element, element_id)
        obj = cls(xml, client)
        obj.id = int(obj.id)
        return obj

    def info(self, *args):
        data = self.client.call(self.METHODS['info'], self.id)
        self._initialize_xml(data, self.ELEMENT_NAME)

    def delete(self):
        '''
        Deletes current object from the pool
        '''
        self.client.call(self.METHODS['delete'], self.id)




# -*- coding: UTF-8 -*-
import os

from mock import Mock

import oca


class TestHostPool:
    def setUp(self):
        self.client = oca.Client('test:test')
        self.xml = open(os.path.join(os.path.dirname(oca.__file__),
                                         'tests/fixtures/hostpool.xml')).read()

    def test_info(self):
        self.client.call = Mock(return_value=self.xml)
        pool = oca.HostPool(self.client)
        pool.info()
        assert len(pool) == 2




# -*- coding: UTF-8 -*-
from pool import Pool, PoolElement, Template, extractString


class Group(PoolElement):
    METHODS = {
        'info'     : 'group.info',
        'allocate' : 'group.allocate',
        'delete'   : 'group.delete',
    }

    XML_TYPES = {
            'id'          : int,
            'name'        : extractString,
            'template'    : ['TEMPLATE', Template],
            'users'       : ['USERS', lambda users: [int(i.text) for i in users]],
            #'resource_providers': handled separately
            #'datastore_quota': handled separately
            #'network_quota': handled separately
            #'vm_quota': handled separately
            #'image_quota'
            #'default_group_quotas'
    }

    ELEMENT_NAME = 'GROUP'

    @staticmethod
    def allocate(client, group_name):
        '''
        Allocates a new group in OpenNebula

        Arguments

        ``client``
           oca.Client object

        ``group``
           a string containing the group name
        '''
        group_id = client.call(Group.METHODS['allocate'], group_name)
        return group_id

    def __init__(self, xml, client):
        super(Group, self).__init__(xml, client)
        self.id = self['ID'] if self['ID'] else None

    def __repr__(self):
        return '<oca.Group("%s")>' % self.name


class GroupPool(Pool):
    METHODS = {
            'info' : 'grouppool.info',
    }

    def __init__(self, client):
        super(GroupPool, self).__init__('GROUP_POOL', 'GROUP', client)

    def _factory(self, xml):
        i = Group(xml, self.client)
        i._convert_types()
        return i




# vim:  set fileencoding=utf8
# encoding:utf8
#
# Author: Matthias Schmitz <matthias@sigxcpu.org>

import unittest
import os
import oca
from oca.exceptions import OpenNebulaException

@unittest.skipUnless(os.environ.has_key('OCA_INT_TESTS'),
                     "Skipping integration tests")
class IntTestTemplate(unittest.TestCase):
    def setUp(self):
        try:
            del os.environ["ONE_AUTH"]
        except KeyError:
            pass

        self.c = oca.Client(os.environ['OCA_INT_TESTS_ONE_AUTH'], os.environ['OCA_INT_TESTS_ONE_XMLRPC'])

    def tearDown(self):
        print "teardown"
        vmp = oca.VirtualMachinePool(self.c)
        vmp.info()
        for vm in vmp:
            if vm.name.startswith('inttest_vm_'):
                vm.delete()

    def test_allocate(self):
        templ = oca.VmTemplate.allocate(self.c, '<VMTEMPLATE><NAME>inttest01</NAME><TEMPLATE/></VMTEMPLATE>')
        templ = oca.VmTemplate.allocate(self.c, '<VMTEMPLATE><NAME>inttest02</NAME><TEMPLATE/></VMTEMPLATE>')
        templ = oca.VmTemplate.allocate(self.c, '<VMTEMPLATE><NAME>inttest03</NAME><TEMPLATE/></VMTEMPLATE>')
        templ = oca.VmTemplate.allocate(self.c, '<VMTEMPLATE><NAME>inttest04</NAME><TEMPLATE/></VMTEMPLATE>')

    def test_allocate_with_same_name(self):
        with self.assertRaises(OpenNebulaException):
            templ = oca.VmTemplate.allocate(self.c, '<VMTEMPLATE><NAME>inttest01</NAME><TEMPLATE/></VMTEMPLATE>')


    def test_update(self):
        oca.VmTemplate.allocate(self.c, '<VMTEMPLATE><NAME>inttest_update01</NAME><TEMPLATE/></VMTEMPLATE>')
        tp = oca.VmTemplatePool(self.c)
        tp.info()
        templ = tp.get_by_name('inttest_update01')
        templ.update('MEMORY=1024 CPU=2')

    def test_delete(self):
        tp = oca.VmTemplatePool(self.c)
        tp.info()
        for tpl in tp:
            if tpl.name.startswith('inttest'):
                tpl.delete()

    def test_instantiate(self):
        templ = oca.VmTemplate.allocate(self.c, '<VMTEMPLATE><NAME>inttest_instantiate_me01</NAME><MEMORY>1234</MEMORY><CPU>2</CPU></VMTEMPLATE>')
        tp = oca.VmTemplatePool(self.c)
        tp.info()
        templ = tp.get_by_name('inttest_instantiate_me01')
        templ.instantiate('inttest_vm_instantiate_me01')
        vmpool = oca.VirtualMachinePool(self.c)
        vmpool.info()
        vm = vmpool.get_by_name('inttest_vm_instantiate_me01')
        self.assertEqual(vm.name, 'inttest_vm_instantiate_me01')




# -*- coding: UTF-8 -*-
from pool import Pool, PoolElement, Template, extractString


class User(PoolElement):
    METHODS = {
        'info':      'user.info',
        'allocate':  'user.allocate',
        'delete':    'user.delete',
        'passwd':    'user.passwd',
        'chgrp':     'user.chgrp'
    }

    XML_TYPES = {
            'id':          int,
            'gid':         int,
            'group_ids':   ['GROUPS', lambda group_ids: map(lambda group_id: int(group_id.text), group_ids)],
            'gname':       extractString,
            'name':        extractString,
            'password':    extractString,
            'auth_driver': extractString,
            'enabled':     bool,
            'template':    ['TEMPLATE', Template],
            #'datastore_quota': handled separately # see http://dev.opennebula.org/issues/3849
            #'network_quota': handled separately   # see http://dev.opennebula.org/issues/3849
            #'vm_quota': handled separately        # see http://dev.opennebula.org/issues/3849
            #'image_quota'                         # see http://dev.opennebula.org/issues/3849
            #'default_user_quotas'                 # see http://dev.opennebula.org/issues/3849
    }

    ELEMENT_NAME = 'USER'

    @staticmethod
    def allocate(client, user, password):
        '''
        allocates a new user in OpenNebula

        ``user``
           username for the new user

        ``password``
           password for the new user
        '''
        user_id = client.call(User.METHODS['allocate'], user, password)
        return user_id

    def __init__(self, xml, client):
        super(User, self).__init__(xml, client)
        self.id = self['ID'] if self['ID'] else None

    def change_passwd(self, new_password):
        '''
        Changes the password for the given user.

        ``new_password``
           The new password
        '''
        self.client.call(User.METHODS['passwd'], self.id, new_password)

    def chgrp(self, gid):
        '''
        Changes the main group

        ``gid``
            New group id. Set to -1 to leave the current one
        '''
        self.client.call(User.METHODS['chgrp'], self.id, gid)

    def __repr__(self):
        return '<oca.User("%s")>' % self.name


class UserPool(Pool):
    METHODS = {
            'info':  'userpool.info',
    }

    def __init__(self, client):
        super(UserPool, self).__init__('USER_POOL', 'USER', client)

    def _factory(self, xml):
        u = User(xml, self.client)
        u._convert_types()
        return u




# -*- coding: UTF-8 -*-
import os

from mock import Mock

import oca


class TestCluster:
    def setUp(self):
        self.client = oca.Client('test:test')
        self.xml = open(os.path.join(os.path.dirname(oca.__file__),
                                     'tests/fixtures/cluster.xml')).read()

    def test_allocate(self):
        self.client.call = Mock(return_value=3)
        assert oca.Cluster.allocate(self.client, 'test') == 3

    def test_repr(self):
        self.client.call = Mock()
        cluster = oca.Cluster(self.xml, self.client)
        assert repr(cluster) == '<oca.Cluster("oneCluster")>'

    def test_convert_types(self):
        cluster = oca.Cluster(self.xml, None)
        cluster._convert_types()
        assert cluster.id == 101
        assert cluster.name == "oneCluster"
        assert cluster.host_ids == [2,3]
        assert cluster.datastore_ids == [4,5]
        assert cluster.vnet_ids == [6,7]
        assert cluster.template.reserved_cpu is None



# -*- coding: UTF-8 -*-
__version__ = '4.10.0-a1'

import os

# from distutils.core import setup
from setuptools import setup

# borrowed from Pylons project
here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.rst')).read()
    CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()
except IOError:
    README = CHANGES = ''

setup(name='oca',
      version=__version__,
      description='Python Bindings for XMLRPC OpenNebula Cloud API',
      long_description=README + '\n\n' + CHANGES,
      test_suite='nose.collector',
      classifiers=['Development Status :: 3 - Alpha',
                   'License :: OSI Approved :: Apache Software License',
                   'License :: OSI Approved :: Apache Software License',
                   'Operating System :: OS Independent'],
      keywords='opennebula cloud xmlrpc',
      author=u'Łukasz Oleś, Matthias Schmitz, Michael Schmidt',
      url='https://github.com/python-oca/python-oca',
      license='Apache License 2.0',
      packages=['oca'])



# -*- coding: UTF-8 -*-
from pool import Pool, PoolElement, Template, extractString


class Image(PoolElement):
    METHODS = {
        'info'     : 'image.info',
        'allocate' : 'image.allocate',
        'delete'   : 'image.delete',
        'update'   : 'image.update',
        'enable'   : 'image.enable',
        'publish'  : 'image.publish',
        'chown'    : 'image.chown',
        'persistent'  : 'image.persistent',
    }

    XML_TYPES = {
            'id'          : int,
            'uid'         : int,
            'gid'         : int,
            'uname'       : extractString,
            'gname'       : extractString,
            'name'        : extractString,
            #'permissions' : ???,
            'type'        : int,
            'disk_type'   : int,
            'persistent'  : int,
            'regtime'     : int,
            'source'      : extractString,
            'path'        : extractString,
            'fstype'      : extractString,
            'size'        : int,
            'state'       : int,
            'running_vms' : int,
            'cloning_ops' : int,
            'cloning_id'  : int,
            'datastore_id': int,
            'datastore'   : extractString,
            'vm_ids'      : ["VMS", lambda vms: map(lambda vm_id: int(vm_id.text), vms)],
            'clone_ids'   : ["CLONES", lambda clones: map(lambda clone_id: int(clone_id.text), clones)],
            'template'    : ['TEMPLATE', Template],
    }

    INIT = 0
    READY = 1
    USED = 2
    DISABLED = 3
    IMAGE_STATES = ['INIT', 'READY', 'USED', 'DISABLED']

    SHORT_IMAGE_STATES = {
            "INIT"      : "init",
            "READY"     : "rdy",
            "USED"      : "used",
            "DISABLED"  : "disa"
    }

    IMAGE_TYPES = ['OS', 'CDROM', 'DATABLOCK']

    SHORT_IMAGE_TYPES = {
            "OS"         : "OS",
            "CDROM"      : "CD",
            "DATABLOCK"  : "DB"
    }

    ELEMENT_NAME = 'IMAGE'

    @staticmethod
    def allocate(client, template):
        '''
        Allocates a new image in OpenNebula

        Arguments

        ``client``
           oca.Client object

        ``template``
           a string containing the template of the image
        '''
        image_id = client.call(Image.METHODS['allocate'], template)
        return image_id

    def __init__(self, xml, client):
        super(Image, self).__init__(xml, client)
        self.id = self['ID'] if self['ID'] else None

    def update(self, template):
        '''
        Replaces the template contents

        Arguments

        ``template``
            New template contents
        '''
        self.client.call(self.METHODS['update'], self.id, template)

    def enable(self):
        '''
        Enables an image
        '''
        self.client.call(self.METHODS['enable'], self.id, True)

    def disable(self):
        '''
        Disables an image
        '''
        self.client.call(self.METHODS['enable'], self.id, False)

    def publish(self):
        '''
        Publishes an image
        '''
        self.client.call(self.METHODS['publish'], self.id, True)

    def unpublish(self):
        '''
        Unpublishes an image
        '''
        self.client.call(self.METHODS['publish'], self.id, False)

    def set_persistent(self):
        '''
        Set Image as persistent
        '''
        self.client.call(self.METHODS['persistent'], self.id, True)

    def set_nonpersistent(self):
        '''
        Set Image as non persistent
        '''
        self.client.call(self.METHODS['persistent'], self.id, False)

    def chown(self, uid, gid):
        '''
        Changes the owner/group

        Arguments

        ``uid``
            New owner id. Set to -1 to leave current value
        ``gid``
            New group id. Set to -1 to leave current value
        '''
        self.client.call(self.METHODS['chown'], self.id, uid, gid)

    @property
    def str_state(self):
        '''
        String representation of image state.
        One of 'INIT', 'READY', 'USED', 'DISABLED'
        '''
        return self.IMAGE_STATES[int(self.state)]

    @property
    def short_state(self):
        '''
        Short string representation of image state.
        One of 'init', 'rdy', 'used', 'disa'
        '''
        return self.SHORT_IMAGE_STATES[self.str_state]

    @property
    def str_type(self):
        '''
        String representation of image type.
        One of 'OS', 'CDROM', 'DATABLOCK'
        '''
        return self.IMAGE_TYPES[int(self.type)]

    @property
    def short_type(self):
        '''
        Short string representation of image type.
        One of 'OS', 'CD', 'DB'
        '''
        return self.SHORT_IMAGE_TYPES[self.str_type]

    def __repr__(self):
        return '<oca.Image("%s")>' % self.name


class ImagePool(Pool):
    METHODS = {
            'info' : 'imagepool.info',
    }

    def __init__(self, client):
        super(ImagePool, self).__init__('IMAGE_POOL', 'IMAGE', client)

    def _factory(self, xml):
        i = Image(xml, self.client)
        i._convert_types()
        return i




# -*- coding: UTF-8 -*-
import os

from mock import Mock

import oca


class TestDatastore:
    def setUp(self):
        self.client = oca.Client('test:test')
        self.xml = open(os.path.join(os.path.dirname(oca.__file__),
                                     'tests/fixtures/datastore.xml')).read()

    def test_allocate(self):
        self.client.call = Mock(return_value=3)
        assert oca.Datastore.allocate(self.client, 'test') == 3

    def test_repr(self):
        self.client.call = Mock()
        datastore = oca.Datastore(self.xml, self.client)
        assert repr(datastore) == '<oca.Datastore("Custom-DS")>'

    def test_convert_types(self):
        cluster = oca.Datastore(self.xml, None)
        cluster._convert_types()
        assert cluster.id == 100
        assert cluster.name == "Custom-DS"
        assert cluster.uid == 0
        assert cluster.gid == 0
        assert cluster.uname == 'oneadmin'
        assert cluster.gname == 'oneadmin'
        assert cluster.ds_mad == 'fs'
        assert cluster.tm_mad == 'ssh'
        assert cluster.base_path == '/var/lib/one//datastores/100'
        assert cluster.type == 0
        assert cluster.disk_type == 0
        assert cluster.cluster_id == -1
        assert cluster.cluster == ""
        assert cluster.total_mb == 9952
        assert cluster.free_mb == 8999
        assert cluster.used_mb == 425
        assert cluster.image_ids == [2,3]
        assert cluster.template.disk_type == 'FILE'



# -*- coding: UTF-8 -*-
from pool import Pool, PoolElement, Template


class VmTemplate(PoolElement):
    METHODS = {
        'info'     : 'template.info',
        'allocate' : 'template.allocate',
        'delete'   : 'template.delete',
        'update'   : 'template.update',
        'publish'  : 'template.publish',
        'chown'    : 'template.chown',
        'instantiate' : 'template.instantiate',
    }

    XML_TYPES = {
            'id'          : int,
            'uid'         : int,
            'gid'         : int,
            'name'        : str,
            'uname'       : str,
            'gname'       : str,
            'regtime'     : int,
            'template'    : ['TEMPLATE', Template]
    }

    ELEMENT_NAME = 'VMTEMPLATE'

    @staticmethod
    def allocate(client, template):
        '''
        Allocates a new template in OpenNebula

        ``client``
           oca.Client object

        ``group``
           a string containing the group name
        '''
        template_id = client.call(VmTemplate.METHODS['allocate'], template)
        return template_id

    def __init__(self, xml, client):
        super(VmTemplate, self).__init__(xml, client)
        self.id = self['ID'] if self['ID'] else None

    def update(self, template):
        '''
        Replaces the template contents.

        ``template``
            The new template contents.
        '''
        self.client.call(VmTemplate.METHODS['update'], self.id, template)

    def publish(self):
        '''
        Publishes a template.
        '''
        self.client.call(VmTemplate.METHODS['publish'], self.id, True)

    def unpublish(self):
        '''
        Unpublishes a template.
        '''
        self.client.call(VmTemplate.METHODS['publish'], self.id, False)

    def chown(self, uid=-1, gid=-1):
        '''
        Changes the ownership of a template.

        ``uid``
            The User ID of the new owner. If set to -1, the owner is not changed.
        ``gid``
             The Group ID of the new group. If set to -1, the group is not changed.
        '''
        self.client.call(VmTemplate.METHODS['chown'], self.id, uid, gid)

    def instantiate(self, name=''):
        '''
        Creates a VM instance from a VmTemplate

        ``name``
            name of the VM instance
        '''
        self.client.call(VmTemplate.METHODS['instantiate'], self.id, name)

    def __repr__(self):
        return '<oca.VmTemplate("%s")>' % self.name


class VmTemplatePool(Pool):
    METHODS = {
            'info' : 'templatepool.info',
    }

    def __init__(self, client):
        super(VmTemplatePool, self).__init__('VMTEMPLATE_POOL', 'VMTEMPLATE', client)

    #def info(self, 

    def _factory(self, xml):
        i = VmTemplate(xml, self.client)
        i._convert_types()
        return i
